####################
# Copyright (c) 2014, Perceptive Automation, LLC. All rights reserved.
# http://www.indigodomo.com
#
# Redistribution of this source file, its binary forms, and images are not allowed.
# We are working on our open source license terms, but at this point the source files,
# its binary forms, and all images cannot be redistributed without written permission
# from both Perceptive Automation, LLC. and Andy Turner, HighEarthOrbit software.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

# RESTful semantics losely based on RoR:
#	http://microformats.org/wiki/rest/urls
#
# GET HTML, XML, and TXT versions of action group list:
#	http://127.0.0.1:8176/actions/
#	http://127.0.0.1:8176/actions.xml/
#	http://127.0.0.1:8176/actions.txt/
#
# GET HTML, XML, and TXT versions of action group "cooking scene":
#	http://127.0.0.1:8176/actions/cooking%20scene.html
#	http://127.0.0.1:8176/actions/cooking%20scene.xml
#	http://127.0.0.1:8176/actions/cooking%20scene.txt
#
# Execute action group "cooking scene":
#	http://127.0.0.1:8176/actions/cooking%20scene?_method=execute
#
# DELETE and POST are not currently supported.

####################
## IMPORTS
import cherrypy
from cherrypy import request
from cherrypy import _cperror

import indigoconn as ic
import indigodb as idb
from basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr
from restreqhandler import RESTRequestHandler

####################
## CONSTANTS

####################
class ActionsRequestHandler(RESTRequestHandler):
	####################
	""" Handles HTTP page requests. """
	def __init__(self, logger):
		RESTRequestHandler.__init__(self, logger)
		self.elemRestParent = u'actions'
		self.elemRestLeaf = u'action'

	exposed = True

	####################
	def _GetElem(self, elem_name):
		return cherrypy.server.indigoDb.GetActionGroup(cherrypy.server.indigoConn, elem_name, True)

	def _GetElemList(self):
		return cherrypy.server.indigoDb.GetActionGroups(cherrypy.server.indigoConn, True, True)

	####################
	def PUT(self, elem_name, format="html", **kw):
		self._CheckRestMimeType(format)
		try:
#			onlyChanges = False		# default to always pushing all passed in parms to server
#			onlyChangesRaw = kw.get('_onlyChanges', None)
#			if onlyChangesRaw is not None:
#				onlyChanges = onlyChangesRaw.lower() in [kTrueStr, u"yes", u"1"]
#
#			blockUntilIdle = True	# default to blocking up to a few seconds while server handles request
#			blockUntilIdleRaw = kw.get('_blockUntilIdle', None)
#			if blockUntilIdleRaw is not None:
#				blockUntilIdle = blockUntilIdleRaw.lower() in [kTrueStr, u"yes", u"1"]

			redirectPath = kw.get('_redirect', cherrypy.url(relative='server'))
# Currently, there are no HTTP client settable attributes.
#			group = cherrypy.server.indigoDb.GetActionGroup(cherrypy.server.indigoConn, elem_name, True)

# No need to block server -- action group attribute changes should be instantaneous.
#			if blockUntilIdle:
#				cherrypy.server.indigoDb.BlockUntilServerIdle(cherrypy.server.indigoConn, cherrypy.server.max_action_change_wait)

			raise cherrypy.HTTPRedirect(redirectPath, 303)
		except (ic.ConnError, ic.InternalError, ic.ServerError), err:
			return self._ReturnError(format, err.value)
		except idb.ControlDisabled:
			return self._ReturnControlDisabled(format)

	####################
	def EXECUTE(self, elem_name, format="html", **kw):
		self._CheckRestMimeType(format)
		try:
			blockUntilIdle = True	# default to blocking up to a few seconds while server handles request
			blockUntilIdleRaw = kw.get('_blockUntilIdle', None)
			if blockUntilIdleRaw is not None:
				blockUntilIdle = blockUntilIdleRaw.lower() in [kTrueStr, u"yes", u"1"]

			redirectPath = kw.get('_redirect', cherrypy.url(relative='server'))
			cherrypy.server.indigoDb.GroupExecute(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)

			if blockUntilIdle:
				cherrypy.server.indigoDb.BlockUntilServerIdle(cherrypy.server.indigoConn, cherrypy.server.max_action_execute_wait)
			raise cherrypy.HTTPRedirect(redirectPath, 303)
		except (ic.ConnError, ic.InternalError, ic.ServerError), err:
			return self._ReturnError(format, err.value)
		except idb.ControlDisabled:
			return self._ReturnControlDisabled(format)
