####################
# Copyright (c) 2014, Perceptive Automation, LLC. All rights reserved.
# http://www.indigodomo.com
#
# Redistribution of this source file, its binary forms, and images are not allowed.
# We are working on our open source license terms, but at this point the source files,
# its binary forms, and all images cannot be redistributed without written permission
# from both Perceptive Automation, LLC. and Andy Turner, HighEarthOrbit software.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

# RESTful semantics losely based on RoR:
#	http://microformats.org/wiki/rest/urls
#	http://community.pachube.com/?q=node/127
#
# GET HTML, XML, and TXT versions of device list:
#	http://127.0.0.1:8176/devices/
#	http://127.0.0.1:8176/devices.xml/
#	http://127.0.0.1:8176/devices.txt/
#
# GET HTML, XML, and TXT versions of device "office-lamp":
#	http://127.0.0.1:8176/devices/office-lamp.html
#	http://127.0.0.1:8176/devices/office-lamp.xml
#	http://127.0.0.1:8176/devices/office-lamp.txt
#
# GET Edit HTML FORM to modify office-lamp editable properties; Form submit will do a virtual PUT to update the properties.
#	http://127.0.0.1:8176/devices/office-lamp/edit
#
# curl and virtual PUT call to change brightness to 27; HTTP result is a 403 redirect back to the device resource after the update is complete.
#	curl -X PUT -d brightness=27 http://127.0.0.1:8176/devices/office-lamp
#	http://127.0.0.1:8176/devices/office-lamp?brightness=27&_method=put
#
# curl and virtual PUT call to change isOn to False and True:
#	curl -X PUT -d isOn=0 http://127.0.0.1:8176/devices/office-lamp
#	curl -X PUT -d isOn=1 http://127.0.0.1:8176/devices/office-lamp
#	http://127.0.0.1:8176/devices/office-lamp?isOn=0&_method=put
#	http://127.0.0.1:8176/devices/office-lamp?isOn=1&_method=put
#
# curl and virtual PUT call to toggle a device's state:
#	curl -X PUT -d toggle=1 http://127.0.0.1:8176/devices/office-lamp
#	http://127.0.0.1:8176/devices/office-lamp?toggle=1&_method=put
#
# curl and virtual PUT call to change speedIndex to 0 (off), 3 (high), then decrease back to 0 (off)
#	curl -X PUT -d speedIndex=0 http://127.0.0.1:8176/devices/office-ceiling-fan
#	curl -X PUT -d speedIndex=3 http://127.0.0.1:8176/devices/office-ceiling-fan
#	curl -X PUT -d speedIndex=dn http://127.0.0.1:8176/devices/office-ceiling-fan
#	curl -X PUT -d speedIndex=dn http://127.0.0.1:8176/devices/office-ceiling-fan
#	curl -X PUT -d speedIndex=dn http://127.0.0.1:8176/devices/office-ceiling-fan
#	http://127.0.0.1:8176/devices/office-ceiling-fan?speedIndex=0&_method=put
#	http://127.0.0.1:8176/devices/office-ceiling-fan?speedIndex=2&_method=put
#	http://127.0.0.1:8176/devices/office-ceiling-fan?speedIndex=3&_method=put
#	http://127.0.0.1:8176/devices/office-ceiling-fan?speedIndex=dn&_method=put
#	http://127.0.0.1:8176/devices/office-ceiling-fan?speedIndex=dn&_method=put
#
# curl and virtual PUT call to change device "irrmaster-pro" active sprinkler zone to 3 and all off:
#	curl -X PUT -d activeZone=3 http://127.0.0.1:8176/devices/irrmaster-pro
#	curl -X PUT -d activeZone=0 http://127.0.0.1:8176/devices/irrmaster-pro
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=3&_method=put
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=0&_method=put
#
# curl and virtual PUT call to change device "irrmaster-pro" sprinkler mode:
#	curl -X PUT -d activeZone=stop http://127.0.0.1:8176/devices/irrmaster-pro
#	curl -X PUT -d activeZone=run http://127.0.0.1:8176/devices/irrmaster-pro
#	curl -X PUT -d activeZone=pause http://127.0.0.1:8176/devices/irrmaster-pro
#	curl -X PUT -d activeZone=resume http://127.0.0.1:8176/devices/irrmaster-pro
#	curl -X PUT -d activeZone=next http://127.0.0.1:8176/devices/irrmaster-pro
#	curl -X PUT -d activeZone=prev http://127.0.0.1:8176/devices/irrmaster-pro
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=stop&_method=put
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=run&_method=put
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=pause&_method=put
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=resume&_method=put
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=next&_method=put
#	http://127.0.0.1:8176/devices/irrmaster-pro?activeZone=prev&_method=put
#
# curl and virtual PUT call to change device "thermostat" heat and cool setpoints:
#	curl -X PUT -d setpointCool=68 -d setpointHeat=84 http://127.0.0.1:8176/devices/thermostat
#	curl -X PUT -d setpointCool=72 -d setpointHeat=78 http://127.0.0.1:8176/devices/thermostat
#	curl -X PUT -d setpointCool=up -d setpointHeat=up http://127.0.0.1:8176/devices/thermostat
#	curl -X PUT -d setpointCool=dn -d setpointHeat=dn http://127.0.0.1:8176/devices/thermostat
#	http://127.0.0.1:8176/devices/thermostat?_method=put&setpointCool=68&setpointHeat=84
#	http://127.0.0.1:8176/devices/thermostat?_method=put&setpointCool=72&setpointHeat=78
#	http://127.0.0.1:8176/devices/thermostat?_method=put&setpointCool=up&setpointHeat=up
#	http://127.0.0.1:8176/devices/thermostat?_method=put&setpointCool=dn&setpointHeat=dn
#
# curl and virtual PUT call to change device "thermostat" hvac mode to "cool on" and "auto on":
#	curl -X PUT -d hvacCurrentMode="cool on" http://127.0.0.1:8176/devices/thermostat
#	curl -X PUT -d hvacCurrentMode="auto on" http://127.0.0.1:8176/devices/thermostat
#	http://127.0.0.1:8176/devices/thermostat?_method=put&hvacCurrentMode=cool%20on
#	http://127.0.0.1:8176/devices/thermostat?_method=put&hvacCurrentMode=auto%20on
#
# curl and virtual PUT call to change device "thermostat" hvac fan mode to "always on" and "auto on":
#	curl -X PUT -d hvacFanMode="always on" http://127.0.0.1:8176/devices/thermostat
#	curl -X PUT -d hvacFanMode="auto on" http://127.0.0.1:8176/devices/thermostat
#	http://127.0.0.1:8176/devices/thermostat?_method=put&hvacFanMode=always%20on
#	http://127.0.0.1:8176/devices/thermostat?_method=put&hvacFanMode=auto%20on
#
# curl and virtual PUT call to change binary output state of an EZIO module device "garage-io":
#	curl -X PUT -d outputBinaryStates="0,0,0,0,1,1,1,1" http://127.0.0.1:8176/devices/garage-io
#	curl -X PUT -d outputBinaryStates="1,0,1,0,0,0,0,0" http://127.0.0.1:8176/devices/garage-io
#	curl -X PUT -d outputBinaryStates="set4" http://127.0.0.1:8176/devices/garage-io
#	curl -X PUT -d outputBinaryStates="clear4" http://127.0.0.1:8176/devices/garage-io
#	http://127.0.0.1:8176/devices/garage-io?_method=put&outputBinaryStates=0,0,0,0,1,1,1,1
#	http://127.0.0.1:8176/devices/garage-io?_method=put&outputBinaryStates=1,0,1,0,0,0,0,0
#	http://127.0.0.1:8176/devices/garage-io?_method=put&outputBinaryStates=set4
#	http://127.0.0.1:8176/devices/garage-io?_method=put&outputBinaryStates=clear4
#
# DELETE and POST are not currently supported.

####################
## IMPORTS
import cherrypy
from cherrypy import request
from cherrypy import _cperror

import indigoconn as ic
import indigoutil as iu
import indigodb as idb
from basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr
from restreqhandler import RESTRequestHandler

####################
## CONSTANTS

####################
class DevicesRequestHandler(RESTRequestHandler):
	####################
	""" Handles HTTP page requests. """
	def __init__(self, logger):
		RESTRequestHandler.__init__(self, logger)
		self.elemRestParent = u'devices'
		self.elemRestLeaf = u'device'

	exposed = True

	####################
	def _GetElem(self, elem_name):
		return cherrypy.server.indigoDb.GetDevice(cherrypy.server.indigoConn, elem_name)

	def _GetElemList(self):
		return cherrypy.server.indigoDb.GetDevices(cherrypy.server.indigoConn, True)

	####################
	def PUT(self, elem_name, format="html", **kw):
		self._CheckRestMimeType(format)
		try:
			onlyChanges = False		# default to always pushing all passed in parms to server
			onlyChangesRaw = kw.get('_onlyChanges', None)
			if onlyChangesRaw is not None:
				onlyChanges = onlyChangesRaw.lower() in [kTrueStr, u"yes", u"1"]

			blockUntilIdle = True	# default to blocking up to a few seconds while server handles request
			blockUntilIdleRaw = kw.get('_blockUntilIdle', None)
			if blockUntilIdleRaw is not None:
				blockUntilIdle = blockUntilIdleRaw.lower() in [kTrueStr, u"yes", u"1"]

			redirectPath = kw.get('_redirect', cherrypy.url(relative='server'))
			device = cherrypy.server.indigoDb.GetDevice(cherrypy.server.indigoConn, elem_name)

			#######
			if device.typeIsDimmer:
				brightness = kw.get('brightness', None)
				if iu.isFloatCompat(brightness):
					brightness = int(round(float(brightness)))
					if not onlyChanges or brightness != device.brightness:
						cherrypy.server.indigoDb.DeviceSetBrightness(cherrypy.server.indigoConn, elem_name, brightness, cherrypy.request.remote.better_ip)

			#######
			if device.typeSupportsOnOff:
				isOn = kw.get('isOn', None)
				if isOn is not None:
					isOn = isOn.lower() in [kTrueStr, u"yes", u"1"]
					if not onlyChanges or isOn != device.isOn:
						if isOn:
							cherrypy.server.indigoDb.DeviceTurnOn(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)
						else:
							cherrypy.server.indigoDb.DeviceTurnOff(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)

				toggle = kw.get('toggle', None)
				if toggle is not None:
					toggle = toggle.lower() in [kTrueStr, u"yes", u"1"]
					if toggle:
						cherrypy.server.indigoDb.DeviceToggleOnOff(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)

			#######
			if device.typeIsSpeedControl:
				speedIndex = kw.get('speedIndex', None)
				if speedIndex is not None:
					if iu.isIntCompat(speedIndex):
						speedIndex = int(speedIndex)
						if not onlyChanges or speedIndex != device.speedIndex:
							cherrypy.server.indigoDb.DeviceSetSpeedIndex(cherrypy.server.indigoConn, elem_name, speedIndex, cherrypy.request.remote.better_ip)
					elif speedIndex.lower() in [u"incr", u"increase", u"up"]:
						cherrypy.server.indigoDb.DeviceDeltaSpeedIndex(cherrypy.server.indigoConn, elem_name, 1, cherrypy.request.remote.better_ip)
					elif speedIndex.lower() in [u"decr", u"decrease", u"dn", u"down"]:
						cherrypy.server.indigoDb.DeviceDeltaSpeedIndex(cherrypy.server.indigoConn, elem_name, -1, cherrypy.request.remote.better_ip)

			#######
			if device.typeIsSprinkler:
				activeZone = kw.get('activeZone', None)
				if activeZone is not None:
					if iu.isIntCompat(activeZone):
						activeZone = int(activeZone)
						if not onlyChanges or activeZone != device.activeZone:
							cherrypy.server.indigoDb.DeviceSetSprinklerActiveZone(cherrypy.server.indigoConn, elem_name, activeZone, cherrypy.request.remote.better_ip)
					else:
						argLower = activeZone.lower()
						if argLower == u"stop":
							cherrypy.server.indigoDb.DeviceSprinklerStopAll(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)
						elif argLower == u"run":
							cherrypy.server.indigoDb.DeviceSprinklerRunPrevious(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)
						elif argLower == u"pause":
							cherrypy.server.indigoDb.DeviceSprinklerPauseSchedule(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)
						elif argLower == u"resume":
							cherrypy.server.indigoDb.DeviceSprinklerResumeSchedule(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)
						elif argLower == u"next":
							cherrypy.server.indigoDb.DeviceSprinklerNextZone(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)
						elif argLower in [u"prev", u"previous"]:
							cherrypy.server.indigoDb.DeviceSprinklerPrevZone(cherrypy.server.indigoConn, elem_name, cherrypy.request.remote.better_ip)

			#######
			if device.typeIsHVAC:
				setpointCool = kw.get('setpointCool', None)
				if setpointCool is not None:
					if iu.isFloatCompat(setpointCool):
						setpointCool = float(setpointCool)
						if not onlyChanges or setpointCool != device.setpointCool:
							cherrypy.server.indigoDb.DeviceSetHvacSetpointCool(cherrypy.server.indigoConn, elem_name, setpointCool, cherrypy.request.remote.better_ip)
					elif setpointCool.lower() in [u"incr", u"increase", u"up"]:
						cherrypy.server.indigoDb.DeviceDeltaHvacSetpointCool(cherrypy.server.indigoConn, elem_name, 1.0, cherrypy.request.remote.better_ip)
					elif setpointCool.lower() in [u"decr", u"decrease", u"dn", u"down"]:
						cherrypy.server.indigoDb.DeviceDeltaHvacSetpointCool(cherrypy.server.indigoConn, elem_name, -1.0, cherrypy.request.remote.better_ip)

				setpointHeat = kw.get('setpointHeat', None)
				if setpointHeat is not None:
					if iu.isFloatCompat(setpointHeat):
						setpointHeat = float(setpointHeat)
						if not onlyChanges or setpointHeat != device.setpointHeat:
							cherrypy.server.indigoDb.DeviceSetHvacSetpointHeat(cherrypy.server.indigoConn, elem_name, setpointHeat, cherrypy.request.remote.better_ip)
					elif setpointHeat.lower() in [u"incr", u"increase", u"up"]:
						cherrypy.server.indigoDb.DeviceDeltaHvacSetpointHeat(cherrypy.server.indigoConn, elem_name, 1.0, cherrypy.request.remote.better_ip)
					elif setpointHeat.lower() in [u"decr", u"decrease", u"dn", u"down"]:
						cherrypy.server.indigoDb.DeviceDeltaHvacSetpointHeat(cherrypy.server.indigoConn, elem_name, -1.0, cherrypy.request.remote.better_ip)

				hvacCurrentMode = kw.get('hvacCurrentMode', None)
				if hvacCurrentMode is not None:
					if iu.isIntCompat(hvacCurrentMode):
						hvacCurrentMode = int(hvacCurrentMode)
					else:									# convert string param to enum int
						hvacCurrentMode = idb.DeviceThermoStrToCurMode(hvacCurrentMode)
					hvacCurrentModeStr = idb.DeviceThermoCurModeToStr(hvacCurrentMode)
					if not onlyChanges or hvacCurrentModeStr != device.hvacCurrentMode:
						cherrypy.server.indigoDb.DeviceSetHvacCurrentMode(cherrypy.server.indigoConn, elem_name, hvacCurrentMode, cherrypy.request.remote.better_ip)

				hvacFanMode = kw.get('hvacFanMode', None)
				if hvacFanMode is not None:
					if iu.isIntCompat(hvacFanMode):
						hvacFanMode = int(hvacFanMode)
					else:									# convert string param to enum int
						hvacFanMode = idb.DeviceThermoStrToFanMode(hvacFanMode)
					hvacFanModeStr = idb.DeviceThermoFanModeToStr(hvacFanMode)
					if not onlyChanges or hvacFanModeStr != device.hvacFanMode:
						cherrypy.server.indigoDb.DeviceSetHvacFanMode(cherrypy.server.indigoConn, elem_name, hvacFanMode, cherrypy.request.remote.better_ip)

			#######
			if device.typeIsMultiIO:
				binStates = kw.get('outputBinaryStates', None)
				if binStates is not None:
					argLower = binStates.lower()
					if argLower.find("set") == 0:
						indexVal = int(argLower.replace("set", ""))
						cherrypy.server.indigoDb.DeviceSetOutputBinaryState(cherrypy.server.indigoConn, elem_name, indexVal, 1, cherrypy.request.remote.better_ip)
					elif argLower.find("clear") == 0:
						indexVal = int(argLower.replace("clear", ""))
						cherrypy.server.indigoDb.DeviceSetOutputBinaryState(cherrypy.server.indigoConn, elem_name, indexVal, 0, cherrypy.request.remote.better_ip)
					elif not onlyChanges or binStates != device.outputBinaryStates:
						cherrypy.server.indigoDb.DeviceSetOutputBinaryStates(cherrypy.server.indigoConn, elem_name, binStates, cherrypy.request.remote.better_ip)

			if blockUntilIdle:
				cherrypy.server.indigoDb.BlockUntilServerIdle(cherrypy.server.indigoConn, cherrypy.server.max_device_change_wait)
			raise cherrypy.HTTPRedirect(redirectPath, 303)
		except (ic.ConnError, ic.InternalError, ic.ServerError), err:
			return self._ReturnError(format, err.value)
		except idb.ControlDisabled:
			return self._ReturnControlDisabled(format)
