####################
# Copyright (c) 2014, Perceptive Automation, LLC. All rights reserved.
# http://www.indigodomo.com
#
# Redistribution of this source file, its binary forms, and images are not allowed.
# We are working on our open source license terms, but at this point the source files,
# its binary forms, and all images cannot be redistributed without written permission
# from both Perceptive Automation, LLC. and Andy Turner, HighEarthOrbit software.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

#	http://community.pachube.com/?q=node/127
#
# RESTful semantics losely based on RoR:
#	http://microformats.org/wiki/rest/urls
#
# GET XML version of insteon links list:
#	http://127.0.0.1:8176/insteonlinks.xml/
#
# GET XML version of links for device "office-lamp":
#	http://127.0.0.1:8176/insteonlinks/office-lamp.xml
#
# PUT to edit insteon links is not currently support.
# DELETE and POST are not currently supported.

####################
## IMPORTS
import cherrypy
from cherrypy import request
from cherrypy import _cperror

import indigoconn as ic
import indigoutil as iu
import indigodb as idb
from basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr
from restreqhandler import RESTRequestHandler

####################
## CONSTANTS

####################
class InsteonLinksRequestHandler(RESTRequestHandler):
	####################
	""" Handles HTTP page requests. """
	def __init__(self, logger):
		RESTRequestHandler.__init__(self, logger)
		self.elemRestParent = u'insteonlinks'
		self.elemRestLeaf = u'insteonlink'

	exposed = True

	####################
	def _GetElem(self, elem_name):
		return cherrypy.server.indigoDb.GetInsteonLinks(cherrypy.server.indigoConn, elem_name)

	def _GetElemList(self):
		fullList = cherrypy.server.indigoDb.GetDevices(cherrypy.server.indigoConn, True)
		insteonOnlyList = []
		for dev in fullList:
			if dev.devProtocol == 1:
				insteonOnlyList.append(dev)
		return insteonOnlyList

	####################
	# Override to always return XML since encoding of the link record list is hard-coded
	# to return XML inside DictElem::_GenerateRawXml() of indigodb.py.
	def GET(self, elem_name="index", format="xml", **kw):
		return RESTRequestHandler.GET(self, elem_name, format="xml", **kw)

	####################
	def PUT(self, elem_name, format="html", **kw):
		self._CheckRestMimeType(format)
		try:
			redirectPath = kw.get('_redirect', cherrypy.url(relative='server'))
			raise cherrypy.HTTPRedirect(redirectPath, 303)
		except (ic.ConnError, ic.InternalError, ic.ServerError), err:
			return self._ReturnError(format, err.value)
		except idb.ControlDisabled:
			return self._ReturnControlDisabled(format)




