####################
# Copyright (c) 2016, Perceptive Automation, LLC. All rights reserved.
# http://www.indigodomo.com
#
# Redistribution of this source file, its binary forms, and images are not allowed.
# We are working on our open source license terms, but at this point the source files,
# its binary forms, and all images cannot be redistributed without written permission
# from both Perceptive Automation, LLC. and Andy Turner, HighEarthOrbit software.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

####################
## IMPORTS
import cherrypy
from cherrypy import request
from cherrypy import _cperror

import indigoconn as ic
import indigodb as idb
from basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr

import urllib2

try:
	import json
except:
	from simplejson import json

####################
## CONSTANTS

####################
class RESTRequestHandler(BaseRequestHandler):
	####################
	""" Handles HTTP page requests. """
	def __init__(self, logger):
		BaseRequestHandler.__init__(self, logger)
		self._EnableRestDispatcher()

	_cp_config = {
		"rest_idname" : "elem_name",
	}
	exposed = True

	####################
	def _GetElem(self, elem_name):
		return None		# subclass most override

	def _GetElemList(self):
		return None		# subclass most override

	####################
	def _FillDefaultTemplateAttrs(self, tmpl):
		BaseRequestHandler._FillDefaultTemplateAttrs(self, tmpl)
		tmpl.elemRestParent = self.elemRestParent
		tmpl.elemRestLeaf = self.elemRestLeaf

	####################
	def GET(self, elem_name="index", format="html", **kw):
		self._CheckRestMimeType(format)
		method_override = self._GetMethodArgOverride("GET", **kw)
		if method_override:
			return method_override(elem_name, format, **kw)
		try:
			cherrypy.response.headers['Content-Type'] = 'text/' + format
			if elem_name == "index":
				elemList = self._GetElemList()
				if format == "json":
					returnList = list()
					for elem in elemList:
						if elem.displayInUI and getattr(elem, "hasStateToDisplay", True):
							shortDict = {
								"name": elem.name,
								"nameURLEncoded": urllib2.quote(elem.name.encode('utf-8')),
								"restParent": self.elemRestParent,
								"restURL": "%s/%s/%s.json" % (cherrypy.request.script_name, self.elemRestParent, urllib2.quote(elem.name.encode('utf-8')))
							}
							returnList.append(shortDict)
					return json.dumps(returnList, indent=4)
				tmpl = self._GetAndLockGlobalTemplate('rest_elemlist.' + str(format))
				try:
					self._FillDefaultTemplateAttrs(tmpl)
					tmpl.elemList = elemList
					return tmpl.RenderTemplate()
				finally:
					tmpl.ReleaseLock()
			else:
				elem = self._GetElem(elem_name)
				if format == "json":
					elemDict = dict(elem)
					elemDict["restParent"] = self.elemRestParent
					return json.dumps(elemDict, indent=4)
				tmpl = self._GetAndLockGlobalTemplate('rest_elem.' + str(format))
				try:
					self._FillDefaultTemplateAttrs(tmpl)
					tmpl.doEdit = False
					tmpl.elem = elem
					tmpl.elemAttrs = elem.StreamPublicKeyVals()
					tmpl.elemIsEditable = elem.IsEditable()
					tmpl.elemTypeName = elem.GetTypeName()
					return tmpl.RenderTemplate()
				finally:
					tmpl.ReleaseLock()
		except (ic.ConnError, ic.InternalError, ic.ServerError), err:
			return self._ReturnError(format, err.value)
		except idb.ControlDisabled:
			return self._ReturnControlDisabled(format)

	####################
	def PUT(self, elem_name, format="html", **kw):
		# Subclasses can override to support REST updating.
		raise cherrypy.HTTPError(405)		# throw Method Not Allowed

	####################
	def POST(self, elem_name, format="html", **kw):
		self._CheckRestMimeType(format)
		method_override = self._GetMethodArgOverride("POST", **kw)
		if method_override:
			return method_override(elem_name, format, **kw)

		# If intended method is really a POST then throw Method Not Allowed,
		# as we currently don't allow any element creation via REST.
		raise cherrypy.HTTPError(405)
		
	####################
	# Returns a form for editing the elem_name record (client then submits via POST)
	def edit(self, elem_name, format="html"):
		if format != "html":
			raise cherrypy.HTTPError(404)
		try:
			elem = self._GetElem(elem_name)
			tmpl = self._GetAndLockGlobalTemplate('rest_elem.' + str(format))
			try:
				self._FillDefaultTemplateAttrs(tmpl)
				tmpl.doEdit = True
				tmpl.elem = elem;
				tmpl.elemAttrs = elem.StreamPublicKeyVals()
				tmpl.elemIsEditable = elem.IsEditable()
				tmpl.elemTypeName = elem.GetTypeName()
				return tmpl.RenderTemplate()
			finally:
				tmpl.ReleaseLock()
		except (ic.ConnError, ic.InternalError, ic.ServerError), err:
			return self._ReturnError(format, err.value)
		except idb.ControlDisabled:
			return self._ReturnControlDisabled(format)
	edit.exposed = True

	####################
	# Return a form for creating a new record (then submitted via POST)
# (not currently supported since we don't allow REST based creation)
#	def new(self, format="html"):
#		return ""
#	new.exposed = True

