####################
# Copyright (c) 2014, Perceptive Automation, LLC. All rights reserved.
# http://www.indigodomo.com
#
# Redistribution of this source file, its binary forms, and images are not allowed.
# We are working on our open source license terms, but at this point the source files,
# its binary forms, and all images cannot be redistributed without written permission
# from both Perceptive Automation, LLC. and Andy Turner, HighEarthOrbit software.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

# RESTful semantics losely based on RoR:
#	http://microformats.org/wiki/rest/urls
#
# GET HTML, XML, and TXT versions of variable list:
#	http://127.0.0.1:8176/variables/
#	http://127.0.0.1:8176/variables.xml/
#	http://127.0.0.1:8176/variables.txt/
#
# GET HTML, XML, and TXT versions of variable "sprinklerDurationMultiplier":
#	http://127.0.0.1:8176/variables/sprinklerDurationMultiplier.html
#	http://127.0.0.1:8176/variables/sprinklerDurationMultiplier.xml
#	http://127.0.0.1:8176/variables/sprinklerDurationMultiplier.txt
#
# curl and virtual PUT call to change variable "sprinklerDurationMultiplier" value to 1.23:
#	curl -X PUT -d value=1.23 http://127.0.0.1:8176/variables/sprinklerDurationMultiplier
#	http://127.0.0.1:8176/variables/sprinklerDurationMultiplier?_method=put&value=1.23
#
# DELETE and POST are not currently supported. Might add them for Variables later.

####################
## IMPORTS
import cherrypy
from cherrypy import request
from cherrypy import _cperror

import indigoconn as ic
import indigodb as idb
from basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr
from restreqhandler import RESTRequestHandler

####################
## CONSTANTS

####################
class VariablesRequestHandler(RESTRequestHandler):
	####################
	""" Handles HTTP page requests. """
	def __init__(self, logger):
		RESTRequestHandler.__init__(self, logger)
		self.elemRestParent = u'variables'
		self.elemRestLeaf = u'variable'

	exposed = True

	####################
	def _GetElem(self, elem_name):
		return cherrypy.server.indigoDb.GetVariable(cherrypy.server.indigoConn, elem_name)

	def _GetElemList(self):
		return cherrypy.server.indigoDb.GetVariables(cherrypy.server.indigoConn, True)

	####################
	def PUT(self, elem_name, format="html", **kw):
		self._CheckRestMimeType(format)
		try:
			onlyChanges = False		# default to always pushing all passed in parms to server
			onlyChangesRaw = kw.get('_onlyChanges', None)
			if onlyChangesRaw is not None:
				onlyChanges = onlyChangesRaw.lower() in [kTrueStr, u"yes", u"1"]

			blockUntilIdle = True	# default to blocking up to a few seconds while server handles request
			blockUntilIdleRaw = kw.get('_blockUntilIdle', None)
			if blockUntilIdleRaw is not None:
				blockUntilIdle = blockUntilIdleRaw.lower() in [kTrueStr, u"yes", u"1"]

			redirectPath = kw.get('_redirect', cherrypy.url(relative='server'))
			variable = cherrypy.server.indigoDb.GetVariable(cherrypy.server.indigoConn, elem_name)

			if not variable.readOnly:
				value = kw.get('value', None)
				if value is not None:
					if not onlyChanges or value != variable.value:
						cherrypy.server.indigoDb.VariableSetValue(cherrypy.server.indigoConn, elem_name, value, cherrypy.request.remote.better_ip)

# No need to block server -- variable changes should be instantaneous.
#			if blockUntilIdle:
#				cherrypy.server.indigoDb.BlockUntilServerIdle(cherrypy.server.indigoConn, cherrypy.server.max_variable_change_wait)
			raise cherrypy.HTTPRedirect(redirectPath, 303)
		except (ic.ConnError, ic.InternalError, ic.ServerError), err:
			return self._ReturnError(format, err.value)
		except idb.ControlDisabled:
			return self._ReturnControlDisabled(format)
